unit MainF;

interface

uses
  Windows, Messages, SysUtils, Forms, KeyCodeU, MWpasU, Registry, IniFiles;

type
  TMainForm = class(TForm)
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
    FConnectAtom    : ATOM;
    FDisconnectAtom : ATOM;
    FEndAtom        : ATOM;
    FInstance       : String;
    FQuitAtom       : ATOM;
    FReconnectAtom  : ATOM;

    procedure WMHotKey(var Msg: TWMHotKey); message WM_HOTKEY;

    procedure RegisterHotkeys;
    procedure UnregisterHotkeys;
    procedure WriteAutostart;
  public
    { Public-Deklarationen }
  end;

var
  MainForm : TMainForm;

implementation

{$R *.dfm}

{ TMainForm }

procedure TMainForm.WMHotKey(var Msg : TWMHotKey);
  procedure SendString(const AString : String);
  var
    LAccess  : TMWconnAccessMode;
    LFile    : THandle;
    LMapping : PMWconnIO;
  begin
    if (IsMWconnRunning(FInstance)) then
    begin
      LAccess := OpenMWconnIO(LFile, LMapping, FInstance);
      try
        if ((LAccess = mwamAll) or (LAccess = mwamWrite)) then
        begin
          ZeroMemory(@LMapping^.Command[0], Length(LMapping^.Command));
          CopyMemory(@LMapping^.Command[0], @AString[1], Length(AString));
        end;
      finally
        CloseMWconnIO(LFile, LMapping);
      end;
    end;
  end;
begin
  if (Msg.HotKey = FConnectAtom) then
    SendString(MWconn_Connect);

  if (Msg.HotKey = FDisconnectAtom) then
    SendString(MWconn_Disconnect);

  if (Msg.HotKey = FEndAtom) then
    SendString(MWconn_End);

  if (Msg.HotKey = FQuitAtom) then
    Close;

  if (Msg.HotKey = FReconnectAtom) then
    SendString(MWconn_Reconnect);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  UnregisterHotkeys;
end;

procedure TMainForm.WriteAutostart;
var
  LRegistry : TRegistry;
begin
  LRegistry := TRegistry.Create(KEY_ALL_ACCESS);
  try
    LRegistry.RootKey := HKEY_CURRENT_USER;

    if LRegistry.OpenKey('Software\Microsoft\Windows\CurrentVersion\Run', false) then
    begin
      try
        LRegistry.WriteString('MWhotkey', Application.ExeName);
      finally
        LRegistry.CloseKey;
      end;
    end;
  finally
    LRegistry.Free;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FConnectAtom    := 0;
  FDisconnectAtom := 0;
  FEndAtom        := 0;
  FQuitAtom       := 0;
  FReconnectAtom  := 0;

  WriteAutostart;
  RegisterHotkeys;
end;

procedure TMainForm.RegisterHotkeys;
  function DoRegister(const AString : String) : ATOM;
  var
    LHotkey : THotkey;
  begin
    Result := 0;

    try
      LHotkey := StringToHotkey(AString);

      Result := GlobalAddAtom(PChar('MWhotkey: ' + LHotkey.Value));
      if (Result > 0) then
        RegisterHotKey(Handle, Result, LHotkey.Modifiers, LHotkey.Key);
    except
      // duplicate key or no key
    end;
  end;
var
  LIniFile : TIniFile;
begin
  UnregisterHotkeys;

  LIniFile := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  try
    FConnectAtom    := DoRegister(LIniFile.ReadString('Hotkey', 'Connect',    'CTRL+ALT+C'));
    FDisconnectAtom := DoRegister(LIniFile.ReadString('Hotkey', 'Disconnect', 'CTRL+ALT+D'));
    FEndAtom        := DoRegister(LIniFile.ReadString('Hotkey', 'End',        'CTRL+ALT+E'));
    FQuitAtom       := DoRegister(LIniFile.ReadString('Hotkey', 'Quit',       'CTRL+ALT+END'));
    FReconnectAtom  := DoRegister(LIniFile.ReadString('Hotkey', 'Reconnect',  'CTRL+ALT+R'));

    FInstance := LIniFile.ReadString('MWconn', 'Instance', '');
  finally
    LIniFile.Free;
  end;
end;

procedure TMainForm.UnregisterHotkeys;
begin
  if (FConnectAtom > 0) then
  begin
    UnRegisterHotKey(Handle, FConnectAtom);
    GlobalDeleteAtom(FConnectAtom);
    FConnectAtom := 0;
  end;

  if (FDisconnectAtom > 0) then
  begin
    UnRegisterHotKey(Handle, FDisconnectAtom);
    GlobalDeleteAtom(FDisconnectAtom);
    FDisconnectAtom := 0;
  end;

  if (FEndAtom > 0) then
  begin
    UnRegisterHotKey(Handle, FEndAtom);
    GlobalDeleteAtom(FEndAtom);
    FEndAtom := 0;
  end;

  if (FQuitAtom > 0) then
  begin
    UnRegisterHotKey(Handle, FQuitAtom);
    GlobalDeleteAtom(FQuitAtom);
    FQuitAtom := 0;
  end;

  if (FReconnectAtom > 0) then
  begin
    UnRegisterHotKey(Handle, FReconnectAtom);
    GlobalDeleteAtom(FReconnectAtom);
    FReconnectAtom := 0;
  end;
end;

end.
